@@ -108,7 +108,7 @@ void nikon_load_raw();
 int nikon_is_compressed();
 int nikon_e995();
 int nikon_e2100();
-int nikon_3700();
+void nikon_3700();
 int minolta_z2();
 void nikon_e900_load_raw();
 void nikon_e2100_load_raw();
@@ -185,7 +185,7 @@ void bilateral_filter();
 void recover_highlights();
 void tiff_get (unsigned base,
 	unsigned *tag, unsigned *type, unsigned *len, unsigned *save);
-void parse_olympus_note (int base);
+void parse_thumb_note (int base, unsigned toff, unsigned tlen);
 void parse_makernote (int base);
 void get_timestamp (int reversed);
 void parse_exif (int base);
