@@ -19,8 +19,10 @@
 #include <errno.h>     /* for errno */
 #include <string.h>
 #include <glib.h>
+#include <glib/gi18n.h>
 #include "ufraw.h"
 
+gboolean silentMessenger;
 char *ufraw_binary;
 
 int main (int argc, char **argv)
@@ -30,6 +32,7 @@ int main (int argc, char **argv)
     int status;
 
     ufraw_binary = g_path_get_basename(argv[0]);
+    uf_init_locale(argv[0]);
 
     /* Load $HOME/.ufrawrc */
     conf_load(&rc, NULL);
@@ -49,6 +52,7 @@ int main (int argc, char **argv)
     int optInd = ufraw_process_args(&argc, &argv, &cmd, &rc);
     if (optInd<0) exit(1);
     if (optInd==0) exit(0);
+    silentMessenger = cmd.silent;
 
     /* Load the --conf file. version==0 means ignore conf. */
     conf.version = 0;
@@ -64,7 +68,7 @@ int main (int argc, char **argv)
 	}
     }
     if (optInd==argc) {
-	    ufraw_message(UFRAW_WARNING, "no input file, nothing to do.");
+	    ufraw_message(UFRAW_WARNING, _("No input file, nothing to do."));
     }
     for (; optInd<argc; optInd++) {
         uf = ufraw_open(argv[optInd]);
@@ -77,9 +81,9 @@ int main (int argc, char **argv)
 
         if (ufraw_load_raw(uf)!=UFRAW_SUCCESS)
             continue;
-        ufraw_message(UFRAW_MESSAGE, "loaded %s", uf->filename);
+        ufraw_message(UFRAW_MESSAGE, _("loaded %s"), uf->filename);
         if (ufraw_batch_saver(uf)==UFRAW_SUCCESS)
-            ufraw_message(UFRAW_MESSAGE, "saved %s",
+            ufraw_message(UFRAW_MESSAGE, _("saved %s"),
 		    uf->conf->outputFilename);
         ufraw_close(uf);
         g_free(uf);
@@ -91,7 +95,7 @@ int main (int argc, char **argv)
 void ufraw_messenger(char *message, void *parentWindow)
 {
     parentWindow = parentWindow;
-    ufraw_batch_messenger(message);
+    if (!silentMessenger) ufraw_batch_messenger(message);
 }
 
 void preview_progress(void *widget, char *text, double progress)
