@@ -47,9 +47,9 @@ extern const char *interpolationNames[];
 enum { no_id, also_id, only_id };
 enum { manual_curve, linear_curve, custom_curve, camera_curve };
 enum { in_profile, out_profile, profile_types};
-enum { raw_expander, exposure_expander, wb_expander, color_expander,
-       curve_expander, live_expander, expander_count };
-enum { ppm8_type, ppm16_type, tiff8_type, tiff16_type, jpeg_type };
+enum { raw_expander, live_expander, expander_count };
+enum { ppm8_type, ppm16_type, tiff8_type, tiff16_type, jpeg_type,
+       embedded_jpeg_type, embedded_png_type };
 
 typedef struct {
     char *make;
@@ -72,8 +72,9 @@ typedef struct {
     double saturation;
     CurveData baseCurveData, luminosityCurveData;
     guint16 gammaCurve[0x10000];
-    guint16 luminosityCurve[0x10000];
-    guint16 saturationCurve[0x10000];
+    void *luminosityProfile;
+    void *TransferFunction[3];
+    void *saturationProfile;
 } developer_data;
 
 typedef guint16 image_type[4];
@@ -141,9 +142,10 @@ typedef struct {
     /* SAVE options */
     char inputFilename[max_path], outputFilename[max_path],
 	 outputPath[max_path];
+    char inputURI[max_path], inputModTime[max_name];
     int type, compression, createID, embedExif;
     int shrink, size;
-    gboolean overwrite, losslessCompress;
+    gboolean overwrite, losslessCompress, embeddedImage;
 
     /* GUI settings */
     double Zoom;
@@ -155,16 +157,20 @@ typedef struct {
     gboolean overExp, underExp;
     char curvePath[max_path];
     char profilePath[max_path];
+    gboolean silent;
 
     /* EXIF data */
+    int flip;
     float iso_speed, shutter, aperture, focal_len;
     char exifSource[max_name], isoText[max_name], shutterText[max_name],
-	 apertureText[max_name], focalLenText[max_name], lensText[max_name];
+	 apertureText[max_name], focalLenText[max_name],
+	 focalLen35Text[max_name], lensText[max_name];
     char timestamp[max_name], make[max_name], model[max_name];
 } conf_data;
 
 typedef struct {
     image_type *image;
+    guint8 *buffer;
     int height, width;
 } image_data;
 
@@ -173,6 +179,7 @@ typedef struct ufraw_struct {
     int predictedHeight, predictedWidth, rgbMax, colors, raw_color, useMatrix;
     float rgb_cam[3][4];
     image_data image;
+    image_data thumb;
     void *raw;
     developer_data *developer;
     conf_data *conf;
@@ -216,6 +223,7 @@ void preview_progress(void *widget, char *text, double progress);
 
 /* prototypes for functions in ufraw_routines.c */
 const char *uf_get_home_dir();
+void uf_init_locale(const char *exename);
 char *uf_file_set_type(const char *filename, const char *type);
 char *uf_file_set_absolute(const char *filename);
 /* Set locale of LC_NUMERIC to "C" to make sure that printf behaves correctly.*/char *uf_set_locale_C();
@@ -257,6 +265,11 @@ long ufraw_saver(void *widget, gpointer user_data);
 int ufraw_write_image(ufraw_data *uf);
 int ufraw_batch_saver(ufraw_data *uf);
 
+/* prototype for functions in ufraw_embedded.c */
+int ufraw_read_embedded(ufraw_data *uf);
+int ufraw_convert_embedded(ufraw_data *uf);
+int ufraw_write_embedded(ufraw_data *uf);
+
 /* prototype for functions in ufraw_chooser.c */
 void ufraw_chooser(conf_data *conf, char *defPath);
 
