@@ -15,11 +15,12 @@
 #endif
 
 #include <string.h>
-#include <ctype.h> /* needed for isspace() */
 #include <errno.h>
 #include <math.h>
+#include <sys/stat.h> /* needed for fstat() */
 #include <getopt.h>
 #include <glib.h>
+#include <glib/gi18n.h>
 #include "ufraw.h"
 
 const conf_data conf_default = {
@@ -36,38 +37,40 @@ const conf_data conf_default = {
     disabled_state, /* autoBlack */
     camera_curve, camera_curve+1, /* BaseCurveIndex, BaseCurveCount */
     /* BaseCurve data defaults */
-    { { "Manual curve", TONE_CURVE, 0.0, 1.0, 0.0, 1.0, 1.0,
+    { { N_("Manual curve"), TONE_CURVE, 0.0, 1.0, 0.0, 1.0, 1.0,
 	  2 , { { 0.0, 0.0 }, { 1.0, 1.0 } } },
-      { "Linear curve", TONE_CURVE, 0.0, 1.0, 0.0, 1.0, 1.0,
+      { N_("Linear curve"), TONE_CURVE, 0.0, 1.0, 0.0, 1.0, 1.0,
 	  2 , { { 0.0, 0.0 }, { 1.0, 1.0 } } },
-      { "Custom curve", TONE_CURVE, 0.0, 1.0, 0.0, 1.0, 1.0,
+      { N_("Custom curve"), TONE_CURVE, 0.0, 1.0, 0.0, 1.0, 1.0,
 	  2 , { { 0.0, 0.0 }, { 1.0, 1.0 } } },
-      { "Camera curve", TONE_CURVE, 0.0, 1.0, 0.0, 1.0, 1.0,
+      { N_("Camera curve"), TONE_CURVE, 0.0, 1.0, 0.0, 1.0, 1.0,
 	  2 , { { 0.0, 0.0 }, { 1.0, 1.0 } } }
     },
     linear_curve, linear_curve+1, /* curveIndex, curveCount */
     /* Curve data defaults */
-    { { "Manual curve", TONE_CURVE, 0.0, 1.0, 0.0, 1.0, 1.0,
+    { { N_("Manual curve"), TONE_CURVE, 0.0, 1.0, 0.0, 1.0, 1.0,
 	  2 , { { 0.0, 0.0 }, { 1.0, 1.0 } } },
-      { "Linear curve", TONE_CURVE, 0.0, 1.0, 0.0, 1.0, 1.0,
+      { N_("Linear curve"), TONE_CURVE, 0.0, 1.0, 0.0, 1.0, 1.0,
 	  2 , { { 0.0, 0.0 }, { 1.0, 1.0 } } }
     },
     { 0, 0 } , { 1, 1 }, /* profileIndex[], profileCount[] */
     /* Profile data defaults */
-    { { { "sRGB", "", "", 0.45, 0.1, TRUE },
+    { { { N_("sRGB"), "", "", 0.45, 0.1, TRUE },
         { "Some ICC Profile", "", "", 0.45, 0.0, FALSE } },
-      { { "sRGB", "", "", 0.0, 0.0, FALSE },
+      { { N_("sRGB"), "", "", 0.0, 0.0, FALSE },
         { "Some ICC Profile", "", "", 0.0, 0.0, FALSE } } },
     0, /* intent */
     ahd_interpolation, /* interpolation */
     "", NULL, /* darkframeFile, darkframe */
     /* Save options */
     "", "", "", /* inputFilename, outputFilename, outputPath */
+    "", "", /* inputURI, inputModTime */
     ppm8_type, 85, no_id, /* type, compression, createID */
     TRUE, /* embedExif */
     1, 0, /* shrink, size */
     FALSE, /* overwrite exsisting files without asking */
     FALSE, /* losslessCompress */
+    FALSE, /* load embedded preview image */
 
     /* GUI settings */
     25.0, 4, /* Zoom, Scale */
@@ -75,13 +78,18 @@ const conf_data conf_default = {
     rgb_histogram, /* histogram */
     linear_histogram, 128, /* liveHistogramScale, liveHistogramHeight */
     linear_histogram, 128, /* rawHistogramScale, rawHistogramHeight */
-    { TRUE, TRUE, TRUE, TRUE, TRUE, TRUE }, /* expander[] */
+    { TRUE, TRUE }, /* expander[] */
     FALSE, /* overExp indicator */
     FALSE, /* underExp indicator */
     "", "", /* curvePath, profilePath */
+    FALSE, /* silent */
+
+    /* EXIF data */
+    0, /* flip */
     0.0, 0.0, 0.0, 0.0, /* iso_speed, shutter, aperture, focal_len */
     "", "", "", "", /* exifSource, isoText, shutterText, apertureText */
-    "", "", "", "", "" /* focalLenText, lensText, timestamp, make, model */
+    "", "", "", /* focalLenText, focalLen35Text, lensText */
+    "", "", "" /* timestamp, make, model */
 };
 
 const char *interpolationNames[] =
@@ -122,18 +130,18 @@ void conf_parse_start(GMarkupParseContext *context, const gchar *element,
         if (!strcmp(element, "UFRaw") && !strcmp(*names, "Version")) {
             if (int_value==3) {
                 ufraw_message(UFRAW_WARNING,
-                    "Trying to convert .ufrawrc from UFRaw-0.4 or earlier");
+                    _("Trying to convert .ufrawrc from UFRaw-0.4 or earlier"));
                 c->version = int_value;
             }
 	    /* In version 7 temperature calculation has changed */
             if (int_value==5) {
                 ufraw_message(UFRAW_WARNING,
-                    "Trying to convert .ufrawrc from UFRaw-0.6 or earlier");
+                   _("Trying to convert .ufrawrc from UFRaw-0.6 or earlier"));
                 c->version = int_value;
             }
             if (int_value!=c->version)
                 g_set_error(error, ufrawQuark, UFRAW_RC_VERSION,
-                    "UFRaw version in .ufrawrc is not supported");
+                    _("UFRaw version in .ufrawrc is not supported"));
         }
         if (!strcmp(*names,"Current") && int_value!=0) {
             if (!strcmp("BaseManualCurve", element))
@@ -243,8 +251,8 @@ void conf_parse_text(GMarkupParseContext *context, const gchar *text, gsize len,
     char temp[max_path];
     int i;
     error = error;
-    for(; len>0 && isspace(*text); len--, text++);
-    for(; len>0 && isspace(text[len-1]); len--);
+    for(; len>0 && g_ascii_isspace(*text); len--, text++);
+    for(; len>0 && g_ascii_isspace(text[len-1]); len--);
     if (len==0) return;
     if (len>max_path-1) len=max_path-1;
     strncpy(temp, text, len);
@@ -264,7 +272,7 @@ void conf_parse_text(GMarkupParseContext *context, const gchar *text, gsize len,
         if (!strcmp("AnchorXY", element)) {
 	    if (c->curve[i].m_numAnchors==max_anchors) {
 		ufraw_message(UFRAW_WARNING,
-			"Too many anchors for curve '%s'", c->curve[i].name);
+			_("Too many anchors for curve '%s'"), c->curve[i].name);
 		/* We try to keep the last anchor point */
 		c->curve[i].m_numAnchors--;
 	    }
@@ -384,14 +392,6 @@ void conf_parse_text(GMarkupParseContext *context, const gchar *text, gsize len,
     }
     if (!strcmp("RawExpander", element))
             sscanf(temp, "%d", &c->expander[raw_expander]);
-    if (!strcmp("ExposureExpander", element))
-            sscanf(temp, "%d", &c->expander[exposure_expander]);
-    if (!strcmp("WBExpander", element))
-            sscanf(temp, "%d", &c->expander[wb_expander]);
-    if (!strcmp("ColorExpander", element))
-            sscanf(temp, "%d", &c->expander[color_expander]);
-    if (!strcmp("CurveExpander", element))
-            sscanf(temp, "%d", &c->expander[curve_expander]);
     if (!strcmp("LiveExpander", element))
             sscanf(temp, "%d", &c->expander[live_expander]);
     if (!strcmp("Histogram", element)) sscanf(temp, "%d", &c->histogram);
@@ -490,19 +490,24 @@ int conf_load(conf_data *c, const char *IDFilename)
     } else {
         if ( (in=fopen(IDFilename, "r"))==NULL ) {
             ufraw_message(UFRAW_SET_ERROR,
-                    "Can't open ID file %s for reading\n%s\n",
+                    _("Can't open ID file %s for reading\n%s\n"),
                     IDFilename, strerror(errno) );
             return UFRAW_ERROR;
         }
 	confFilename = g_strdup(IDFilename);
     }
+    g_snprintf(c->inputURI, max_path, "file://%s", confFilename);
+    struct stat s;
+    fstat(fileno(in), &s);
+    g_snprintf(c->inputModTime, max_name, "%d", (int)s.st_mtime);
+
     locale = uf_set_locale_C();
     context = g_markup_parse_context_new(&parser, 0, c, NULL);
     line[max_path-1] = '\0';
     fgets(line, max_path-1, in);
     while (!feof(in)) {
         if (!g_markup_parse_context_parse(context, line, strlen(line), &err)) {
-            ufraw_message(UFRAW_ERROR, "Error parsing '%s'\n%s",
+            ufraw_message(UFRAW_ERROR, _("Error parsing '%s'\n%s"),
                     confFilename, err->message);
 	    g_markup_parse_context_free(context);
 	    uf_reset_locale(locale);
@@ -586,20 +591,6 @@ int conf_save(conf_data *c, char *IDFilename, char **confBuffer)
         if (c->expander[raw_expander]!=conf_default.expander[raw_expander])
             buf = uf_markup_buf(buf, "<RawExpander>%d</RawExpander>\n",
                     c->expander[raw_expander]);
-        if (c->expander[exposure_expander]!=
-		conf_default.expander[exposure_expander])
-            buf = uf_markup_buf(buf,
-		    "<ExposureExpander>%d</ExposureExpander>\n",
-                    c->expander[exposure_expander]);
-        if (c->expander[wb_expander]!=conf_default.expander[wb_expander])
-            buf = uf_markup_buf(buf, "<WBExpander>%d</WBExpander>\n",
-		    c->expander[wb_expander]);
-        if (c->expander[color_expander]!=conf_default.expander[color_expander])
-            buf = uf_markup_buf(buf, "<ColorExpander>%d</ColorExpander>\n",
-                    c->expander[color_expander]);
-        if (c->expander[curve_expander]!=conf_default.expander[curve_expander])
-            buf = uf_markup_buf(buf, "<CurveExpander>%d</CurveExpander>\n",
-                    c->expander[curve_expander]);
         if (c->expander[live_expander]!=conf_default.expander[live_expander])
             buf = uf_markup_buf(buf, "<LiveExpander>%d</LiveExpander>\n",
                     c->expander[live_expander]);
@@ -831,17 +822,17 @@ int conf_save(conf_data *c, char *IDFilename, char **confBuffer)
 	    buf = uf_markup_buf(buf,
 		    "<DarkframeFile>%s</DarkframeFile>\n", c->darkframeFile);
 	buf = uf_markup_buf(buf, "<Timestamp>%s</Timestamp>\n", c->timestamp);
-	buf = uf_markup_buf(buf, "<ISOSpeed>%d</ISOSpeed>\n",
-		(int)c->isoText);
+	buf = uf_markup_buf(buf, "<ISOSpeed>%s</ISOSpeed>\n", c->isoText);
 	buf = uf_markup_buf(buf, "<Shutter>%s</Shutter>\n", c->shutterText);
 	buf = uf_markup_buf(buf, "<Aperture>%s</Aperture>\n", c->apertureText);
 	buf = uf_markup_buf(buf, "<FocalLength>%s</FocalLength>\n",
-		(int)c->focalLenText);
+		c->focalLenText);
+	buf = uf_markup_buf(buf, "<FocalLength35>%s</FocalLength35>\n",
+		c->focalLen35Text);
 	if (strlen(c->lensText)>0)
-	    buf = uf_markup_buf(buf, "<Lens>%s</Lens>\n",
-		    (int)c->lensText);
+	    buf = uf_markup_buf(buf, "<Lens>%s</Lens>\n", c->lensText);
 	buf = uf_markup_buf(buf, "<EXIFSource>%s</EXIFSource>\n",
-		(int)c->exifSource);
+		c->exifSource);
 	char *log = ufraw_message(UFRAW_GET_LOG, NULL);
 	if (log!=NULL) {
 	    char *utf8 = g_filename_to_utf8(log, -1, NULL, NULL, NULL);
@@ -863,7 +854,7 @@ int conf_save(conf_data *c, char *IDFilename, char **confBuffer)
 	    confFilename = g_strdup(IDFilename);
         if ( (out=fopen(confFilename, "w"))==NULL ) {
             ufraw_message(UFRAW_ERROR,
-                    "Can't open file %s for writing\n%s\n",
+                    _("Can't open file %s for writing\n%s\n"),
                     confFilename, strerror(errno) );
 	    g_free(confFilename);
 	    g_free(buf);
@@ -989,6 +980,8 @@ void conf_copy_save(conf_data *dst, const conf_data *src)
     g_strlcpy(dst->outputFilename, src->outputFilename, max_path);
     g_strlcpy(dst->outputPath, src->outputPath, max_path);
     */
+    g_strlcpy(dst->inputURI, src->inputURI, max_path);
+    g_strlcpy(dst->inputModTime, src->inputModTime, max_name);
     dst->type = src->type;
     dst->compression = src->compression;
     dst->createID = src->createID;
@@ -997,6 +990,7 @@ void conf_copy_save(conf_data *dst, const conf_data *src)
     dst->size = src->size;
     dst->overwrite = src->overwrite;
     dst->losslessCompress = src->losslessCompress;
+    dst->embeddedImage = src->embeddedImage;
 }
 
 int conf_set_cmd(conf_data *conf, const conf_data *cmd)
@@ -1006,6 +1000,8 @@ int conf_set_cmd(conf_data *conf, const conf_data *cmd)
     if (cmd->losslessCompress!=-1)
 	conf->losslessCompress = cmd->losslessCompress;
     if (cmd->embedExif!=-1) conf->embedExif = cmd->embedExif;
+    if (cmd->embeddedImage!=-1) conf->embeddedImage = cmd->embeddedImage;
+    if (cmd->silent!=-1) conf->silent = cmd->silent;
     if (cmd->compression!=NULLF) conf->compression = cmd->compression;
     if (cmd->autoExposure) {
 	conf->autoExposure = cmd->autoExposure;
@@ -1059,9 +1055,9 @@ int conf_set_cmd(conf_data *conf, const conf_data *cmd)
     if (strlen(cmd->outputPath)>0)
 	g_strlcpy(conf->outputPath, cmd->outputPath, max_path);
     if (strlen(cmd->outputFilename)>0) {
-        if (conf->createID!=no_id && !strcmp(cmd->outputFilename,"-")) {
-            ufraw_message(UFRAW_ERROR,
-                "cannot --create-id with stdout");
+        if ( conf->createID!=no_id && !strcmp(cmd->outputFilename,"-") &&
+	     !cmd->embeddedImage ) {
+            ufraw_message(UFRAW_ERROR, _("cannot --create-id with stdout"));
             return UFRAW_ERROR;
         }
 	g_strlcpy(conf->outputFilename, cmd->outputFilename, max_path);
@@ -1074,79 +1070,83 @@ int conf_set_cmd(conf_data *conf, const conf_data *cmd)
  * closely related to the other configuration functions. */
 
 
-char helpText[]=
-"UFRaw " VERSION " - Unidentified Flying Raw converter for digital camera images.\n"
-"\n"
-"Usage: ufraw [ options ... ] [ raw-image-files ... ]\n"
-"       ufraw-batch [ options ... ] [ raw-image-files ... ]\n"
-"       ufraw [ options ... ] [ default-directory ]\n"
-"\n"
-"By default 'ufraw' displays a preview window for each raw image allowing\n"
+char *helpText[] = {
+N_("UFRaw "), VERSION, N_(" - Unidentified Flying Raw converter for digital camera images.\n"),
+"\n",
+N_("Usage: ufraw [ options ... ] [ raw-image-files ... ]\n"),
+N_("       ufraw-batch [ options ... ] [ raw-image-files ... ]\n"),
+N_("       ufraw [ options ... ] [ default-directory ]\n"),
+"\n",
+N_("By default 'ufraw' displays a preview window for each raw image allowing\n"
 "the user to tweak the image parameters before saving. If no raw images\n"
 "are given at the command line, UFRaw will display a file chooser dialog.\n"
 "To process the images with no questions asked (and no preview) use\n"
-"'ufraw-batch'.\n"
-"\n"
-"The input files can be either raw images or ufraw's ID files. ID file\n"
+"'ufraw-batch'.\n"),
+"\n",
+N_("The input files can be either raw images or ufraw's ID files. ID file\n"
 "contain a raw image filename and the parameters for handling the image.\n"
-"One can also use and ID file with the option:\n"
-"\n"
-"--conf=ID-file        Apply the parameters in ID-file to other raw images.\n"
-"\n"
-"The rest of the options are separated into two groups.\n"
-"The options which are related to the image manipulation are:\n"
-"\n"
-"--wb=camera|auto      White balance setting.\n"
-"--temperature=TEMP    Color temperature in Kelvins (2000 - 7000).\n"
-"--green=GREEN         Green color normalization.\n"
-"--base-curve=manual|linear|camera|custom|CURVE\n"
-"                      Type of base tone curve to use. CURVE can be any curve\n"
-"                      that was previously loaded in the GUI.\n"
-"                      (default camera if such exsists, linear otherwise).\n"
-"--base-curve-file=file\n"
+"One can also use an ID file with the option:\n"),
+"\n",
+N_("--conf=ID-file        Apply the parameters in ID-file to other raw images.\n"),
+"\n",
+N_("The rest of the options are separated into two groups.\n"),
+N_("The options which are related to the image manipulation are:\n"),
+"\n",
+N_("--wb=camera|auto      White balance setting.\n"),
+N_("--temperature=TEMP    Color temperature in Kelvins (2000 - 7000).\n"),
+N_("--green=GREEN         Green color normalization.\n"),
+N_("--base-curve=manual|linear|camera|custom|CURVE\n"
+"                      Type of base tone curve to use. CURVE can be any\n"
+"                      curve that was previously loaded in the GUI.\n"
+"                      (default camera if such exsists, linear otherwise).\n"),
+N_("--base-curve-file=file\n"
 "                      Use base tone curve included in specified file.\n"
-"                      Overrides --base-curve option.\n"
-"--curve=manual|linear|CURVE\n"
+"                      Overrides --base-curve option.\n"),
+N_("--curve=manual|linear|CURVE\n"
 "                      Type of luminosity curve to use. CURVE can be any\n"
 "                      curve that was previously loaded in the GUI.\n"
-"                      (default linear).\n"
-"--curve-file=file     Use luminosity curve included in specified file.\n"
-"                      Overrides --curve option.\n"
-"--[un]clip            Unclip [clip] highlights (default clip).\n"
-"--gamma=GAMMA         Gamma adjustment of the base curve (default 0.45).\n"
-"--linearity=LINEARITY Linearity of the base curve (default 0.10).\n"
-"--saturation=SAT      Saturation adjustment (default 1.0, 0 for B&W output).\n"
-"--exposure=auto|EXPOSURE\n"
-"                      Auto exposure or exposure correction in EV (default 0).\n"
-"--black-point=auto|BLACK\n"
-"                      Auto black-point or black-point value (default 0).\n"
-"--interpolation=ahd|vng|four-color|bilinear\n"
-"                      Interpolation algorithm to use (default ahd).\n"
-"\n"
-"The options which are related to the final output are:\n"
-"\n"
-"--shrink=FACTOR       Shrink the image by FACTOR (default 1).\n"
-"--size=SIZE           Downsize max(height,width) to SIZE.\n"
-"--out-type=ppm8|ppm16|tiff8|tiff16|jpeg\n"
-"                      Output file format (default ppm8).\n"
-"--create-id=no|also|only\n"
-"                      Create no|also|only ID file (default no).\n"
-"--compression=VALUE   JPEG compression (0-100, default 85).\n"
-"--[no]exif            Embed exif in output JPEG (default embed exif).\n"
-"--[no]zip             Enable [disable] TIFF zip compression (default nozip).\n"
-"--out-path=PATH       PATH for output file (default use input file's path).\n"
-"--output=FILE         Output file name, use '-' to output to stdout.\n"
-"--darkframe=FILE      Use FILE for raw darkframe subtraction.\n"
-"--overwrite           Overwrite existing files without asking (default no).\n"
-"\n"
-"UFRaw first reads the setting from the resource file $HOME/.ufrawrc.\n"
+"                      (default linear).\n"),
+N_("--curve-file=file     Use luminosity curve included in specified file.\n"
+"                      Overrides --curve option.\n"),
+N_("--[un]clip            Unclip [clip] highlights (default clip).\n"),
+N_("--gamma=GAMMA         Gamma adjustment of the base curve (default 0.45).\n"),
+N_("--linearity=LINEARITY Linearity of the base curve (default 0.10).\n"),
+N_("--saturation=SAT      Saturation adjustment (default 1.0, 0 for B&W output).\n"),
+N_("--exposure=auto|EXPOSURE\n"
+"                      Auto exposure or exposure correction in EV (default 0).\n"),
+N_("--black-point=auto|BLACK\n"
+"                      Auto black-point or black-point value (default 0).\n"),
+N_("--interpolation=ahd|vng|four-color|bilinear\n"
+"                      Interpolation algorithm to use (default ahd).\n"),
+"\n",
+N_("The options which are related to the final output are:\n"),
+"\n",
+N_("--shrink=FACTOR       Shrink the image by FACTOR (default 1).\n"),
+N_("--size=SIZE           Downsize max(height,width) to SIZE.\n"),
+N_("--out-type=ppm8|ppm16|tiff8|tiff16|jpeg\n"
+"                      Output file format (default ppm8).\n"),
+N_("--create-id=no|also|only\n"
+"                      Create no|also|only ID file (default no).\n"),
+N_("--compression=VALUE   JPEG compression (0-100, default 85).\n"),
+N_("--[no]exif            Embed exif in output JPEG (default embed exif).\n"),
+N_("--[no]zip             Enable [disable] TIFF zip compression (default nozip).\n"),
+N_("--embedded-image      Extract the preview image embedded in the raw file\n"
+"                      instead of converting the raw image.\n"),
+N_("--out-path=PATH       PATH for output file (default use input file's path).\n"),
+N_("--output=FILE         Output file name, use '-' to output to stdout.\n"),
+N_("--darkframe=FILE      Use FILE for raw darkframe subtraction.\n"),
+N_("--overwrite           Overwrite existing files without asking (default no).\n"),
+"\n",
+N_("UFRaw first reads the setting from the resource file $HOME/.ufrawrc.\n"
 "Then, if an ID file is specified it setting are read. Next, the setting from\n"
 "the --conf option are taken, ignoring input/output filenames in the ID file.\n"
 "Lastly, the options from the command line are set. In batch mode, the second\n"
-"group of options is NOT read from the resource file.\n"
-"\n"
-"Last, but not least, --version displays the version number and compilation\n"
-"options for ufraw and --help displays this help message and exits.\n";
+"group of options is NOT read from the resource file.\n"),
+"\n",
+N_("Last, but not least, --version displays the version number and compilation\n"
+"options for ufraw and --help displays this help message and exits.\n"),
+"END"
+};
 
 char versionText[]= 
 "%s " VERSION "\n" 
@@ -1232,6 +1232,8 @@ int ufraw_process_args(int *argc, char ***argv, conf_data *cmd, conf_data *rc)
         {"overwrite", 0, 0, 'O'},
 	{"exif", 0, 0, 'E'},
 	{"noexif", 0, 0, 'F'},
+	{"embedded-image", 0, 0, 'm'},
+	{"silent", 0, 0, 'q'},
         {"help", 0, 0, 'h'},
 	{"version", 0, 0, 'v'},
 	{"batch", 0, 0, 'b'},
@@ -1250,6 +1252,8 @@ int ufraw_process_args(int *argc, char ***argv, conf_data *cmd, conf_data *rc)
     cmd->losslessCompress=-1;
     cmd->overwrite=-1;
     cmd->embedExif=-1;
+    cmd->embeddedImage=FALSE;
+    cmd->silent=FALSE;
     cmd->profile[0][0].gamma=NULLF;
     cmd->profile[0][0].linear=NULLF;
     cmd->saturation=NULLF;
@@ -1283,7 +1287,7 @@ int ufraw_process_args(int *argc, char ***argv, conf_data *cmd, conf_data *rc)
         case 'd':
             if (sscanf(optarg, "%lf", (double *)optPointer[index])==0) {
                 ufraw_message(UFRAW_ERROR,
-			"'%s' is not a valid value for the --%s option.",
+			_("'%s' is not a valid value for the --%s option."),
 			optarg, options[index].name);
                 return -1;
             }
@@ -1293,7 +1297,7 @@ int ufraw_process_args(int *argc, char ***argv, conf_data *cmd, conf_data *rc)
         case 'j':
             if (sscanf(optarg, "%d", (int *)optPointer[index])==0) {
                 ufraw_message(UFRAW_ERROR,
-			"'%s' is not a valid value for the --%s option.",
+			_("'%s' is not a valid value for the --%s option."),
 			optarg, options[index].name);
                 return -1;
             }
@@ -1315,19 +1319,24 @@ int ufraw_process_args(int *argc, char ***argv, conf_data *cmd, conf_data *rc)
         case 'u': cmd->unclip = TRUE; break;
         case 'U': cmd->unclip = FALSE; break;
         case 'O': cmd->overwrite = TRUE; break;
+	case 'm': cmd->embeddedImage = TRUE; break;
+	case 'q': cmd->silent = TRUE; break;
         case 'z':
 #ifdef HAVE_LIBZ
             cmd->losslessCompress = TRUE; break;
 #else
             ufraw_message(UFRAW_ERROR,
-                "ufraw was build without ZIP support.");
+                _("ufraw was build without ZIP support."));
             return -1;
 #endif
         case 'Z': cmd->losslessCompress = FALSE; break;
         case 'E': cmd->embedExif = TRUE; break;
         case 'F': cmd->embedExif = FALSE; break;
         case 'h':
-            ufraw_message(UFRAW_WARNING, helpText);
+	    for (i=0; strcmp(helpText[i],"END")!=0; i++)
+		ufraw_message(UFRAW_SET_WARNING, _(helpText[i]));
+	    ufraw_message(UFRAW_WARNING,
+		    ufraw_message(UFRAW_GET_WARNING, NULL));
             return 0;
 	case 'v':
 	    base = g_path_get_basename(*argv[0]);
@@ -1336,7 +1345,7 @@ int ufraw_process_args(int *argc, char ***argv, conf_data *cmd, conf_data *rc)
             return 0;
 	case 'b':
             ufraw_message(UFRAW_ERROR,
-                "--batch is obselete. use ufraw-batch instead.");
+                _("--batch is obselete. use ufraw-batch instead."));
             return -1;
         case '?': /* invalid option. Warning printed by getopt() */
             return -1;
@@ -1354,13 +1363,12 @@ int ufraw_process_args(int *argc, char ***argv, conf_data *cmd, conf_data *rc)
 	    g_strlcpy(cmd->wb, auto_wb, max_name);
         else {
             ufraw_message(UFRAW_ERROR,
-                    "'%s' is not a valid white balance setting.", wbName);
+                    _("'%s' is not a valid white balance setting."), wbName);
             return -1;
         }
 	if (cmd->temperature!=NULLF || cmd->green!=NULLF) {
             ufraw_message(UFRAW_WARNING,
-		    "--temperature and --green options override "
-		    "the --wb=%s option.", wbName);
+		    _("--temperature and --green options override the --wb=%s option."), wbName);
 	    g_strlcpy(cmd->wb, "", max_name);
 	}
     }
@@ -1368,15 +1376,14 @@ int ufraw_process_args(int *argc, char ***argv, conf_data *cmd, conf_data *rc)
     if (baseCurveFile!=NULL) {
         if (cmd->BaseCurveCount == max_curves) {
             ufraw_message(UFRAW_ERROR,
-		    "failed to load curve from %s,"
-		    "too many configured base curves", baseCurveFile);
+		    _("failed to load curve from %s, too many configured base curves"), baseCurveFile);
             return -1;
         }
         cmd->BaseCurveIndex = cmd->BaseCurveCount;
         if (curve_load(&(rc->BaseCurve[cmd->BaseCurveIndex]), baseCurveFile)
 		== UFRAW_ERROR) {
             ufraw_message(UFRAW_ERROR,
-                    "failed to load curve from %s", baseCurveFile);
+                    _("failed to load curve from %s"), baseCurveFile);
             return -1;
         }
         cmd->BaseCurveCount++;
@@ -1399,7 +1406,7 @@ int ufraw_process_args(int *argc, char ***argv, conf_data *cmd, conf_data *rc)
             }
             if( cmd->BaseCurveIndex == -1 ) {
                 ufraw_message(UFRAW_ERROR,
-                        "'%s' is not a valid base curve name.", baseCurveName);
+                        _("'%s' is not a valid base curve name."), baseCurveName);
                 return -1;
             }
         }
@@ -1408,15 +1415,14 @@ int ufraw_process_args(int *argc, char ***argv, conf_data *cmd, conf_data *rc)
     if (curveFile!=NULL) {
         if (cmd->curveCount == max_curves) {
             ufraw_message(UFRAW_ERROR,
-		    "failed to load curve from %s,"
-		    "too many configured curves", curveFile);
+		    _("failed to load curve from %s, too many configured curves"), curveFile);
             return -1;
         }
         cmd->curveIndex = cmd->curveCount;
         if (curve_load(&(rc->curve[cmd->curveIndex]), curveFile) == UFRAW_ERROR)
 	{
             ufraw_message(UFRAW_ERROR,
-                    "failed to load curve from %s", curveFile);
+                    _("failed to load curve from %s"), curveFile);
             return -1;
         }
         cmd->curveCount++;
@@ -1433,7 +1439,7 @@ int ufraw_process_args(int *argc, char ***argv, conf_data *cmd, conf_data *rc)
             }
             if( cmd->curveIndex == -1 ) {
                 ufraw_message(UFRAW_ERROR,
-                        "'%s' is not a valid curve name.", curveName);
+                        _("'%s' is not a valid curve name."), curveName);
                 return -1;
             }
         }
@@ -1449,18 +1455,18 @@ int ufraw_process_args(int *argc, char ***argv, conf_data *cmd, conf_data *rc)
 		    interpolationNames);
 	if (cmd->interpolation<0 || cmd->interpolation==half_interpolation) {
             ufraw_message(UFRAW_ERROR,
-                "'%s' is not a valid interpolation option.",
+                _("'%s' is not a valid interpolation option."),
                 interpolationName);
             return -1;
         }
     }
     if (cmd->shrink!=NULLF && cmd->size!=NULLF) {
         ufraw_message(UFRAW_ERROR,
-                        "you can not specify both --shrink and --size");
+                        _("you can not specify both --shrink and --size"));
         return -1;
     }
     cmd->type = -1;
-    if (outTypeName!=NULL) {
+    if (outTypeName!=NULL && !cmd->embeddedImage) {
         if (!strcmp(outTypeName, "ppm8"))
             cmd->type = ppm8_type;
         else if (!strcmp(outTypeName, "ppm16"))
@@ -1471,7 +1477,7 @@ int ufraw_process_args(int *argc, char ***argv, conf_data *cmd, conf_data *rc)
 #else
         {
             ufraw_message(UFRAW_ERROR,
-		    "ufraw was build without TIFF support.");
+		    _("ufraw was build without TIFF support."));
             return -1;
         }
 #endif
@@ -1481,7 +1487,7 @@ int ufraw_process_args(int *argc, char ***argv, conf_data *cmd, conf_data *rc)
 #else
         {
             ufraw_message(UFRAW_ERROR,
-		    "ufraw was build without TIFF support.");
+		    _("ufraw was build without TIFF support."));
             return -1;
         }
 #endif
@@ -1491,13 +1497,31 @@ int ufraw_process_args(int *argc, char ***argv, conf_data *cmd, conf_data *rc)
 #else
         {
             ufraw_message(UFRAW_ERROR,
-		    "ufraw was build without JPEG support.");
+		    _("ufraw was build without JPEG support."));
             return -1;
         }
 #endif
         else {
             ufraw_message(UFRAW_ERROR,
-		    "'%s' is not a valid output type.", outTypeName);
+		    _("'%s' is not a valid output type."), outTypeName);
+            return -1;
+        }
+    }
+    if (cmd->embeddedImage) {
+#ifndef HAVE_LIBJPEG
+	ufraw_message(UFRAW_ERROR, _("ufraw was build without JPEG support."));
+	return -1;
+#endif
+        if ( outTypeName==NULL || strcmp(outTypeName, "jpeg")==0 )
+	    cmd->type = embedded_jpeg_type;
+#ifdef HAVE_LIBPNG
+        else if ( strcmp(outTypeName, "png")==0 )
+	    cmd->type = embedded_png_type;
+#endif
+	else {
+            ufraw_message(UFRAW_ERROR,
+		    _("'%s' is not a valid output type for embedded image."),
+		    outTypeName);
             return -1;
         }
     }
@@ -1511,7 +1535,7 @@ int ufraw_process_args(int *argc, char ***argv, conf_data *cmd, conf_data *rc)
             cmd->createID = only_id;
         else {
             ufraw_message(UFRAW_ERROR,
-		    "'%s' is not a valid create-id option.", createIDName);
+		    _("'%s' is not a valid create-id option."), createIDName);
             return -1;
         }
     }
@@ -1520,20 +1544,21 @@ int ufraw_process_args(int *argc, char ***argv, conf_data *cmd, conf_data *rc)
         if (g_file_test(outPath, G_FILE_TEST_IS_DIR))
             g_strlcpy(cmd->outputPath, outPath, max_path);
         else {
-            ufraw_message(UFRAW_ERROR, "'%s' is not a valid path.", outPath);
+            ufraw_message(UFRAW_ERROR, _("'%s' is not a valid path."), outPath);
             return -1;
         }
     }
     g_strlcpy(cmd->outputFilename, "", max_path);
     if (output!=NULL) {
         if (*argc-optind>1) {
-            ufraw_message(UFRAW_ERROR, "cannot output more than "
-                                "one file to the same output");
+            ufraw_message(UFRAW_ERROR,
+		    _("cannot output more than one file to the same output"));
             return -1;
         }
         g_strlcpy(cmd->outputFilename, output, max_path);
     }
     g_strlcpy(cmd->darkframeFile, "", max_path);
+    cmd->darkframe = NULL;
     if (darkframeFile!=NULL) {
 	char *df = uf_file_set_absolute(darkframeFile);
 	cmd->darkframe = ufraw_load_darkframe(df);
