@@ -19,10 +19,10 @@
 #endif
 #include <stdlib.h> /* needed for canonicalize_file_name() */
 #include <string.h>
-#include <ctype.h> /* needed for isspace() */
 #include <errno.h>
 #include <locale.h>
 #include <glib.h>
+#include <glib/gi18n.h>
 #include "ufraw.h"
 
 /* we start by some general purpose functions that mostly take care of
@@ -40,6 +40,47 @@ const char *uf_get_home_dir()
     return hd;
 }
 
+void uf_init_locale(const char *exename)
+{
+    const char *locale = setlocale(LC_ALL, "");
+    /* Disable the Hebrew and Arabic locale, since the right-to-left setting
+     * does not go well with the preview window. */
+    if ( locale!=NULL &&
+	(!strncmp(locale, "he", 2) || !strncmp(locale, "iw", 2) ||
+	!strncmp(locale, "ar", 2) ||
+	!strncmp(locale, "Hebrew", 6) || !strncmp(locale, "Arabic", 6) ) ) {
+	/* I'm not sure why the following doesn't work (on Windows at least) */
+	/* locale = setlocale(LC_ALL, "C");
+	 * gtk_disable_setlocale(); */
+	/* so I'm using setenv */
+	g_setenv("LC_ALL", "C", TRUE);
+    }
+    /* Try getting the localedir from the environment */
+    char *localedir = g_strconcat(g_getenv("UFRAW_LOCALEDIR"), NULL);
+    if (localedir==NULL) {
+        /* If that fails, there are two defaults: */
+#ifdef WIN32
+	/* In Windows the localedir is found relative to the exe file.
+	 * The exact location here should match ufraw-setup.iss.in */
+	if ( strcasecmp(g_path_get_basename(exename), "ufraw-gimp.exe")==0 ) {
+	    localedir = g_strconcat(g_path_get_dirname(exename),
+		    "/../../../locale", NULL);
+	} else {
+	    localedir = g_strconcat(g_path_get_dirname(exename),
+		    "/../lib/locale", NULL);
+	}
+#else
+	exename = exename; /* suppress warning */
+	/* In other environments localedir is set at compile time */
+	localedir = g_strconcat(UFRAW_LOCALEDIR, NULL);
+#endif
+    }
+    bindtextdomain("ufraw", localedir);
+    g_free(localedir);
+    bind_textdomain_codeset("ufraw", "UTF-8");
+    textdomain("ufraw");
+}
+
 char *uf_file_set_type(const char *filename, const char *type)
 {
     char *file, *cp;
@@ -100,7 +141,8 @@ char *uf_markup_buf(char *buffer, const char *format, ...)
 const char raw_ext[]= "arw,bay,bmq,cr2,crw,cs1,dc2,dcr,dng,erf,fff,hdr,jpg,k25,"
 	"kdc,mdc,mos,mrw,nef,orf,pef,pxn,raf,raw,rdc,sr2,srf,sti,tif,ufraw,x3f";
 
-const char *file_type[] = { ".ppm", ".ppm", ".tif", ".tif", ".jpg" };
+const char *file_type[] = { ".ppm", ".ppm", ".tif", ".tif", ".jpg",
+	".embedded.jpg", ".embedded.png" };
 
 /* Set locale of LC_NUMERIC to "C" to make sure that printf behaves correctly.*/
 char *uf_set_locale_C()
@@ -223,7 +265,7 @@ void curve_parse_start(GMarkupParseContext *context, const gchar *element,
 	     * previous versions */
             if (int_value>conf_default.version)
                 g_set_error(error, ufrawQuark, UFRAW_RC_VERSION,
-                    "Curve version is not supported");
+                    _("Curve version is not supported"));
         }
         names++;
         values++;
@@ -257,8 +299,8 @@ void curve_parse_text(GMarkupParseContext *context, const gchar *text,
     const gchar *element = g_markup_parse_context_get_element(context);
     char temp[max_path];
     error = error;
-    for(; len>0 && isspace(*text); len--, text++);
-    for(; len>0 && isspace(text[len-1]); len--);
+    for(; len>0 && g_ascii_isspace(*text); len--, text++);
+    for(; len>0 && g_ascii_isspace(text[len-1]); len--);
     if (len==0) return;
     if (len>max_path-1) len=max_path-1;
     strncpy(temp, text, len);
@@ -290,7 +332,7 @@ int curve_load(CurveData *cp, char *filename)
 	 !strcasecmp(filename + strlen(filename) - 4, ".ncv") ) {
         /* Try loading ntc/ncv files */
         if (LoadNikonData(filename, &data)!=UFRAW_SUCCESS) {
-            ufraw_message(UFRAW_ERROR, "Invalid Nikon curve file '%s'",
+            ufraw_message(UFRAW_ERROR, _("Invalid Nikon curve file '%s'"),
                         filename);
             return UFRAW_ERROR;
         }
@@ -306,7 +348,7 @@ int curve_load(CurveData *cp, char *filename)
 
         *cp = conf_default.curve[0];
         if ( (in=fopen(filename, "r"))==NULL ) {
-            ufraw_message(UFRAW_ERROR, "Error opening Curve file '%s': %s",
+            ufraw_message(UFRAW_ERROR, _("Error opening Curve file '%s': %s"),
 			    filename, strerror(errno));
 	    return UFRAW_ERROR;
 	}
@@ -317,7 +359,7 @@ int curve_load(CurveData *cp, char *filename)
         while (!feof(in)) {
             if (!g_markup_parse_context_parse(context, line,
                     strlen(line), &err)) {
-                ufraw_message(UFRAW_ERROR, "Error parsing '%s'\n%s",
+                ufraw_message(UFRAW_ERROR, _("Error parsing '%s'\n%s"),
                         filename, err->message);
 		g_markup_parse_context_free(context);
 		uf_reset_locale(locale);
@@ -360,27 +402,20 @@ int curve_save(CurveData *cp, char *filename)
         //clear it out
         memset(&data,0,sizeof(NikonData));
 
-        //check to see if we can save
-        if (cp->m_numAnchors < 2) {
-            ufraw_message(UFRAW_ERROR, "Error saving Nikon curve file."
-                        "Curve data is not from an NEF, NTC, or NCV file.");
-            return UFRAW_ERROR;
-        }
 	data.curves[TONE_CURVE] = *cp;
 
         if (SaveNikonDataFile(&data, filename, nikon_file_type,
                                 NIKON_VERSION_4_1)!=UFRAW_SUCCESS) {
-            ufraw_message(UFRAW_ERROR, "Invalid Nikon curve file '%s'",
+            ufraw_message(UFRAW_ERROR, _("Invalid Nikon curve file '%s'"),
                             filename);
             return UFRAW_ERROR;
         }
-//        FreeNikonData(&data);
     } else {
         /* Save UFRaw's curve format */
         FILE *out;
 
         if ( (out=fopen(filename, "w"))==NULL ) {
-            ufraw_message(UFRAW_ERROR, "Error opening file '%s': %s",
+            ufraw_message(UFRAW_ERROR, _("Error opening file '%s': %s"),
                         filename, g_strerror(errno));
             return UFRAW_ERROR;
         }
