@@ -17,10 +17,12 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <sys/stat.h> /* for fstat() */
 #include <math.h>
 #include <time.h>
 #include <errno.h>
 #include <glib.h>
+#include <glib/gi18n.h>
 #include "dcraw_api.h"
 #include "nikon_curve.h"
 #include "ufraw.h"
@@ -131,7 +133,7 @@ ufraw_data *ufraw_open(char *filename)
     fname = g_filename_from_uri(filename, &hostname, NULL);
     if (fname!=NULL) {
 	if (hostname!=NULL) {
-	    ufraw_message(UFRAW_SET_ERROR, "Remote URI is not supported");
+	    ufraw_message(UFRAW_SET_ERROR, _("Remote URI is not supported"));
 	    g_free(hostname);
 	    g_free(fname);
 	    return NULL;
@@ -148,6 +150,31 @@ ufraw_data *ufraw_open(char *filename)
 	    return NULL;
 	}
 	if (conf->createID==only_id) conf->createID = also_id;
+
+	/* If inputFilename and outputFilename have the same path,
+	 * then inputFilename is searched for in the path of the ID file.
+	 * This allows moving raw and ID files together between folders. */
+	char *inPath = g_path_get_dirname(conf->inputFilename);
+	char *outPath = g_path_get_dirname(conf->outputFilename);
+	if ( strcmp(inPath, outPath)==0 ) {
+	    char *path = g_path_get_dirname(filename);
+	    char *inName = g_path_get_basename(conf->inputFilename);
+	    char *inFile = g_strconcat(path, "/", inName, NULL);
+	    if ( g_file_test(inFile, G_FILE_TEST_EXISTS) ) {
+		g_strlcpy(conf->inputFilename, inFile, max_path);
+	    }
+	    g_free(path);
+	    g_free(inName);
+	    g_free(inFile);
+	}
+	g_free(inPath);
+	g_free(outPath);
+
+	/* Output image should be created in the path of the ID file */
+	char *path = g_path_get_dirname(filename);
+	g_strlcpy(conf->outputPath, path, max_path);
+	g_free(path);
+
 	filename = conf->inputFilename;
     }
     raw = g_new(dcraw_data, 1);
@@ -162,6 +189,7 @@ ufraw_data *ufraw_open(char *filename)
     uf->conf = conf;
     g_strlcpy(uf->filename, filename, max_path);
     uf->image.image = NULL;
+    uf->thumb.buffer = NULL;
     uf->raw = raw;
     uf->colors = raw->colors;
     uf->raw_color = raw->raw_color;
@@ -193,7 +221,8 @@ ufraw_data *ufraw_load_darkframe(char *darkframeFile)
         uf = ufraw_open(darkframeFile);
         if (!uf){
             ufraw_message(UFRAW_ERROR,
-		    "darkframe error: %s is not a raw file\n", darkframeFile);
+		    _("darkframe error: %s is not a raw file\n"),
+		    darkframeFile);
             return NULL;
         }
 	uf->conf = g_new(conf_data, 1);
@@ -202,12 +231,12 @@ ufraw_data *ufraw_load_darkframe(char *darkframeFile)
 	uf->conf->autoExposure = disabled_state;
 	uf->conf->autoBlack = disabled_state;
         if (ufraw_load_raw(uf)==UFRAW_SUCCESS){
-            ufraw_message(UFRAW_BATCH_MESSAGE, "using darkframe %s\n",
+            ufraw_message(UFRAW_BATCH_MESSAGE, _("using darkframe '%s'\n"),
 		    uf->filename);
             return uf;
         }
         else
-            ufraw_message(UFRAW_ERROR, "error loading darkframe %s\n",
+            ufraw_message(UFRAW_ERROR, _("error loading darkframe '%s'\n"),
 		    uf->filename);
     }
     return NULL;
@@ -215,7 +244,6 @@ ufraw_data *ufraw_load_darkframe(char *darkframeFile)
 
 int ufraw_config(ufraw_data *uf, conf_data *rc, conf_data *conf, conf_data *cmd)
 {
-    dcraw_data *raw=NULL;
     int status;
     gboolean loadingID = FALSE;
 
@@ -230,6 +258,7 @@ int ufraw_config(ufraw_data *uf, conf_data *rc, conf_data *conf, conf_data *cmd)
 	    conf_data tmp = *rc;
 	    conf_copy_image(&tmp, uf->conf);
 	    conf_copy_save(&tmp, uf->conf);
+	    g_strlcpy(tmp.outputPath, uf->conf->outputPath, max_path);
 	    *uf->conf = tmp;
 	} else {
 	    uf->conf = g_new(conf_data, 1);
@@ -250,18 +279,21 @@ int ufraw_config(ufraw_data *uf, conf_data *rc, conf_data *conf, conf_data *cmd)
     }
     if (uf==NULL) return UFRAW_SUCCESS;
 
+    dcraw_data *raw = uf->raw;
     char *absname = uf_file_set_absolute(uf->filename);
     g_strlcpy(uf->conf->inputFilename, absname, max_path);
     g_free(absname);
-
+    if (!loadingID) {
+	g_snprintf(uf->conf->inputURI, max_path, "file://%s",
+		uf->conf->inputFilename);
+	struct stat s;
+	fstat(fileno(raw->ifp), &s);
+	g_snprintf(uf->conf->inputModTime, max_name, "%d", (int)s.st_mtime);
+    }
     if (ufraw_exif_from_raw(uf)!=UFRAW_SUCCESS) {
         ufraw_message(UFRAW_SET_LOG, "Error reading EXIF data from %s\n",
                 uf->filename);
     }
-    /* There is a memory leak if 'raw' is set before ufraw_exif_from_raw().
-     * I have not found the cause, but the solution is to set it here. */
-    raw = uf->raw;
-
     /* If we switched cameras, ignore channel multipliers and
      * change spot_wb to manual_wb */
     if ( !loadingID &&
@@ -284,6 +316,7 @@ int ufraw_config(ufraw_data *uf, conf_data *rc, conf_data *conf, conf_data *cmd)
 	uf->conf->timestamp[strlen(uf->conf->timestamp)-1] = '\0';
     g_strlcpy(uf->conf->make, raw->make, max_name);
     g_strlcpy(uf->conf->model, raw->model, max_name);
+    uf->conf->flip = raw->flip;
     if (uf->exifBuf==NULL) {
 	g_strlcpy(uf->conf->exifSource, "DCRaw", max_name);
 	uf->conf->iso_speed = raw->iso_speed;
@@ -299,8 +332,8 @@ int ufraw_config(ufraw_data *uf, conf_data *rc, conf_data *conf, conf_data *cmd)
 	g_snprintf(uf->conf->apertureText, max_name, "F/%0.1f",
 		uf->conf->aperture);
 	uf->conf->focal_len = raw->focal_len;
-	g_snprintf(uf->conf->focalLenText, max_name, "%d mm",
-		(int)uf->conf->focal_len);
+	g_snprintf(uf->conf->focalLenText, max_name, "%0.1f mm",
+		uf->conf->focal_len);
     }
     /* Always set useMatrix if colors=4
      * Always reset useMatrix if 'raw_color' */
@@ -314,7 +347,7 @@ int ufraw_config(ufraw_data *uf, conf_data *rc, conf_data *conf, conf_data *cmd)
 	long pos = ftell(raw->ifp);
 	if (RipNikonNEFCurve(raw->ifp, raw->toneCurveOffset, &nc, NULL)
 		!=UFRAW_SUCCESS) {
-	    ufraw_message(UFRAW_ERROR, "Error reading NEF curve");
+	    ufraw_message(UFRAW_ERROR, _("Error reading NEF curve"));
 	    return UFRAW_WARNING;
 	}
 	fseek(raw->ifp, pos, SEEK_SET);
@@ -376,7 +409,8 @@ void ufraw_substract_darkframe(ufraw_data *uf)
         org->raw.width!=df->raw.width &&
         org->raw.colors!=df->raw.colors){
 
-        ufraw_message(UFRAW_SET_WARNING, "incompatible darkframe");
+        ufraw_message(UFRAW_SET_WARNING,
+		_("Darkframe is incompatible with main image"));
         return;
     }
 
@@ -400,6 +434,16 @@ int ufraw_load_raw(ufraw_data *uf)
     int status;
     dcraw_data *raw = uf->raw;
 
+    if (uf->conf->embeddedImage) {
+	dcraw_image_data thumb;
+	if ( (status=dcraw_load_thumb(raw, &thumb))!=DCRAW_SUCCESS ) {
+	    ufraw_message(status, raw->message);
+	    return status;
+	}
+	uf->thumb.height = thumb.height;
+	uf->thumb.width = thumb.width;
+	return ufraw_read_embedded(uf);
+    }
     if ( (status=dcraw_load_raw(raw))!=DCRAW_SUCCESS ) {
         ufraw_message(status, raw->message);
         return status;
@@ -423,6 +467,7 @@ void ufraw_close(ufraw_data *uf)
     g_free(uf->raw);
     g_free(uf->exifBuf);
     g_free(uf->image.image);
+    g_free(uf->thumb.buffer);
     developer_destroy(uf->developer);
     g_free(uf->RawLumHistogram);
     ufraw_message(UFRAW_CLEAN, NULL);
@@ -485,7 +530,7 @@ int ufraw_convert_image(ufraw_data *uf)
     }
     if (uf->conf->size>0) {
         if ( uf->conf->size>MAX(final.height, final.width) ) {
-            ufraw_message(UFRAW_ERROR, "Can not downsize from %d to %d.",
+            ufraw_message(UFRAW_ERROR, _("Can not downsize from %d to %d."),
                     MAX(final.height, final.width), uf->conf->size);
         } else {
             dcraw_image_resize(&final, uf->conf->size);
@@ -584,11 +629,11 @@ int ufraw_set_wb(ufraw_data *uf)
 		!strcmp(uf->conf->wb, camera_wb)))!=DCRAW_SUCCESS ) {
             if (status==DCRAW_NO_CAMERA_WB) {
                 ufraw_message(UFRAW_BATCH_MESSAGE,
-                    "Cannot use camera white balance, "
-                    "reverting to auto white balance.");
+                    _("Cannot use camera white balance, "
+                    "reverting to auto white balance."));
                 ufraw_message(UFRAW_INTERACTIVE_MESSAGE,
-                    "Cannot use camera white balance, "
-                    "reverting to auto white balance.");
+                    _("Cannot use camera white balance, "
+                    "reverting to auto white balance."));
 		g_strlcpy(uf->conf->wb, auto_wb, max_name);
 		return ufraw_set_wb(uf);
             }
